@@ -43,6 +43,12 @@ type Reader struct {
 	done             *atomic.Bool
 	highlightingDone *atomic.Bool
 
+	highlightingStyle chan chroma.Style
+
+	// This channel expects to be read exactly once. All other uses will lead to
+	// undefined behavior.
+	doneWaitingForFirstByte chan bool
+
 	// For telling the UI it should recheck the --quit-if-one-screen conditions.
 	// Signalled when either highlighting is done or reading is done.
 	maybeDone chan bool
@@ -54,7 +60,7 @@ type Reader struct {
 type InputLines struct {
 	lines []*Line
 
-	// One-based line number of the first line returned
+	// Line number of the first line returned
 	firstLine linenumbers.LineNumber
 
 	// "monkey.txt: 1-23/45 51%"
@@ -119,22 +125,32 @@ func (reader *Reader) readStream(stream io.Reader, originalFileName *string, onD
 		var err error
 		for keepReadingLine {
 			lineBytes, keepReadingLine, err = bufioReader.ReadLine()
-			if err != nil {
-				if err == io.EOF {
-					eof = true
-					break
+
+			if err == nil {
+				// Async write, we probably already wrote to it during the last
+				// iteration
+				select {
+				case reader.doneWaitingForFirstByte <- true:
+				default:
 				}
 
-				reader.Lock()
-				if reader.err == nil {
-					// Store the error unless it overwrites one we already have
-					reader.err = fmt.Errorf("error reading line from input stream: %w", err)
-				}
-				reader.Unlock()
+				completeLine = append(completeLine, lineBytes...)
+				continue
+			}
+
+			// Something went wrong
+
+			if err == io.EOF {
+				eof = true
 				break
 			}
 
-			completeLine = append(completeLine, lineBytes...)
+			reader.Lock()
+			if reader.err == nil {
+				// Store the error unless it overwrites one we already have
+				reader.err = fmt.Errorf("error reading line from input stream: %w", err)
+			}
+			reader.Unlock()
 		}
 
 		if eof {
@@ -167,6 +183,14 @@ func (reader *Reader) readStream(stream io.Reader, originalFileName *string, onD
 		}
 	}
 
+	// If the stream was empty we never got any first byte. Make sure people
+	// stop waiting in this case. Async write since it might already have been
+	// written to.
+	select {
+	case reader.doneWaitingForFirstByte <- true:
+	default:
+	}
+
 	if onDone != nil {
 		onDone()
 	}
@@ -174,14 +198,10 @@ func (reader *Reader) readStream(stream io.Reader, originalFileName *string, onD
 	log.Debug("Stream read in ", time.Since(t0))
 }
 
-// NewReaderFromStream creates a new stream reader
-//
-// The name can be an empty string ("").
-//
-// If non-empty, the name will be displayed by the pager in the bottom left
-// corner to help the user keep track of what is being paged.
-func NewReaderFromStream(name string, reader io.Reader, style chroma.Style, formatter chroma.Formatter, lexer chroma.Lexer) *Reader {
-	mReader := newReaderFromStream(reader, nil, style, formatter, lexer)
+// Note that you must call reader.SetStyleForHighlighting() after this to get
+// highlighting.
+func NewReaderFromStreamWithoutStyle(name string, reader io.Reader, formatter chroma.Formatter, lexer chroma.Lexer) *Reader {
+	mReader := newReaderFromStream(reader, nil, formatter, lexer)
 
 	if len(name) > 0 {
 		mReader.Lock()
@@ -196,6 +216,18 @@ func NewReaderFromStream(name string, reader io.Reader, style chroma.Style, form
 	return mReader
 }
 
+// NewReaderFromStream creates a new stream reader
+//
+// The name can be an empty string ("").
+//
+// If non-empty, the name will be displayed by the pager in the bottom left
+// corner to help the user keep track of what is being paged.
+func NewReaderFromStream(name string, reader io.Reader, style chroma.Style, formatter chroma.Formatter, lexer chroma.Lexer) *Reader {
+	mReader := NewReaderFromStreamWithoutStyle(name, reader, formatter, lexer)
+	mReader.SetStyleForHighlighting(style)
+	return mReader
+}
+
 // newReaderFromStream creates a new stream reader
 //
 // originalFileName is used for counting the lines in the file. nil for
@@ -204,7 +236,10 @@ func NewReaderFromStream(name string, reader io.Reader, style chroma.Style, form
 // loading performance.
 //
 // If lexer is not nil, the file will be highlighted after being fully read.
-func newReaderFromStream(reader io.Reader, originalFileName *string, style chroma.Style, formatter chroma.Formatter, lexer chroma.Lexer) *Reader {
+//
+// Note that you must call reader.SetStyleForHighlighting() after this to get
+// highlighting.
+func newReaderFromStream(reader io.Reader, originalFileName *string, formatter chroma.Formatter, lexer chroma.Lexer) *Reader {
 	done := atomic.Bool{}
 	done.Store(false)
 	highlightingDone := atomic.Bool{}
@@ -213,17 +248,19 @@ func newReaderFromStream(reader io.Reader, originalFileName *string, style chrom
 		// This needs to be size 1. If it would be 0, and we add more
 		// lines while the pager is processing, the pager would miss
 		// the lines added while it was processing.
-		moreLinesAdded:   make(chan bool, 1),
-		maybeDone:        make(chan bool, 1),
-		highlightingDone: &highlightingDone,
-		done:             &done,
+		moreLinesAdded:          make(chan bool, 1),
+		maybeDone:               make(chan bool, 1),
+		highlightingStyle:       make(chan chroma.Style, 1),
+		doneWaitingForFirstByte: make(chan bool, 1),
+		highlightingDone:        &highlightingDone,
+		done:                    &done,
 	}
 
 	// FIXME: Make sure that if we panic somewhere inside of this goroutine,
 	// the main program terminates and prints our panic stack trace.
 	go returnMe.readStream(reader, originalFileName, func() {
 		if lexer != nil {
-			highlightFromMemory(&returnMe, style, formatter, lexer)
+			highlightFromMemory(&returnMe, <-returnMe.highlightingStyle, formatter, lexer)
 		}
 	})
 
@@ -251,9 +288,10 @@ func NewReaderFromText(name string, text string) *Reader {
 	highlightingDone := atomic.Bool{}
 	highlightingDone.Store(true) // No highlighting to do = nothing left = Done!
 	returnMe := &Reader{
-		lines:            lines,
-		done:             &done,
-		highlightingDone: &highlightingDone,
+		lines:                   lines,
+		done:                    &done,
+		highlightingDone:        &highlightingDone,
+		doneWaitingForFirstByte: make(chan bool, 1),
 	}
 	if name != "" {
 		returnMe.name = &name
@@ -338,14 +376,9 @@ func countLines(filename string) (uint64, error) {
 	return count, nil
 }
 
-// NewReaderFromFilename creates a new file reader.
-//
-// If lexer is nil it will be determined from the input file name.
-//
-// The Reader will try to uncompress various compressed file format, and also
-// apply highlighting to the file using Chroma:
-// https://github.com/alecthomas/chroma
-func NewReaderFromFilename(filename string, style chroma.Style, formatter chroma.Formatter, lexer chroma.Lexer) (*Reader, error) {
+// Note that you must call reader.SetStyleForHighlighting() after this to get
+// highlighting.
+func NewReaderFromFilenameWithoutStyle(filename string, formatter chroma.Formatter, lexer chroma.Lexer) (*Reader, error) {
 	fileError := tryOpen(filename)
 	if fileError != nil {
 		return nil, fileError
@@ -359,18 +392,34 @@ func NewReaderFromFilename(filename string, style chroma.Style, formatter chroma
 	// Set lexer to nil in this call since we want to do our own highlighting in
 	// parallel with the stream being read. See the call to
 	// StartHighlightingFromFile() below.
-	returnMe := newReaderFromStream(stream, &filename, chroma.Style{}, nil, nil)
+	returnMe := newReaderFromStream(stream, &filename, nil, nil)
 
 	returnMe.Lock()
 	returnMe.name = &filename
 	returnMe.Unlock()
 
-	startHighlightingFromFile(returnMe, filename, style, formatter, lexer)
+	startHighlightingFromFile(returnMe, filename, formatter, lexer)
 
 	return returnMe, nil
 }
 
-func startHighlightingFromFile(reader *Reader, filename string, style chroma.Style, formatter chroma.Formatter, lexer chroma.Lexer) {
+// NewReaderFromFilename creates a new file reader.
+//
+// If lexer is nil it will be determined from the input file name.
+//
+// The Reader will try to uncompress various compressed file format, and also
+// apply highlighting to the file using Chroma:
+// https://github.com/alecthomas/chroma
+func NewReaderFromFilename(filename string, style chroma.Style, formatter chroma.Formatter, lexer chroma.Lexer) (*Reader, error) {
+	mReader, err := NewReaderFromFilenameWithoutStyle(filename, formatter, lexer)
+	if err != nil {
+		return nil, err
+	}
+	mReader.SetStyleForHighlighting(style)
+	return mReader, nil
+}
+
+func startHighlightingFromFile(reader *Reader, filename string, formatter chroma.Formatter, lexer chroma.Lexer) {
 	reportDone := func() {
 		reader.highlightingDone.Store(true)
 		select {
@@ -407,7 +456,7 @@ func startHighlightingFromFile(reader *Reader, filename string, style chroma.Sty
 			lexer = lexers.Match(filename)
 		}
 
-		highlighted, err := highlight(string(fileBytes), style, formatter, lexer)
+		highlighted, err := highlight(string(fileBytes), <-reader.highlightingStyle, formatter, lexer)
 		if err != nil {
 			log.Warn("Highlighting failed: ", err)
 			return
@@ -422,6 +471,7 @@ func startHighlightingFromFile(reader *Reader, filename string, style chroma.Sty
 	}()
 }
 
+// We expect this to be executed in a goroutine
 func highlightFromMemory(reader *Reader, style chroma.Style, formatter chroma.Formatter, lexer chroma.Lexer) {
 	if lexer == nil {
 		return
@@ -492,6 +542,14 @@ func (reader *Reader) createStatusUnlocked(lastLine linenumbers.LineNumber) stri
 		percent)
 }
 
+// Wait for the first line to be read.
+//
+// Used for making sudo work:
+// https://github.com/walles/moar/issues/199
+func (reader *Reader) AwaitFirstByte() {
+	<-reader.doneWaitingForFirstByte
+}
+
 // GetLineCount returns the number of lines available for viewing
 func (reader *Reader) GetLineCount() int {
 	reader.Lock()
@@ -561,6 +619,51 @@ func (reader *Reader) getLinesUnlocked(firstLine linenumbers.LineNumber, wantedL
 		overflow
 }
 
+func (reader *Reader) PumpToStdout() {
+	const wantedLineCount = 100
+	firstNotPrintedLine := linenumbers.LineNumberFromOneBased(1)
+
+	drainLines := func() bool {
+		lines, _ := reader.GetLines(firstNotPrintedLine, wantedLineCount)
+
+		// Print the lines we got
+		printed := false
+		for index, line := range lines.lines {
+			lineNumber := lines.firstLine.NonWrappingAdd(index)
+			if lineNumber.IsBefore(firstNotPrintedLine) {
+				continue
+			}
+
+			fmt.Println(line.raw)
+			printed = true
+			firstNotPrintedLine = lineNumber.NonWrappingAdd(1)
+		}
+
+		return printed
+	}
+
+	drainAllLines := func() {
+		for drainLines() {
+			// Loop here until nothing was printed
+		}
+	}
+
+	done := false
+	for !done {
+		drainAllLines()
+
+		select {
+		case <-reader.moreLinesAdded:
+			continue
+		case <-reader.maybeDone:
+			done = true
+		}
+	}
+
+	// Print any remaining lines
+	drainAllLines()
+}
+
 // Replace reader contents with the given text and mark as done
 func (reader *Reader) setText(text string) {
 	lines := []*Line{}
@@ -592,3 +695,7 @@ func (reader *Reader) setText(text string) {
 	default:
 	}
 }
+
+func (reader *Reader) SetStyleForHighlighting(style chroma.Style) {
+	reader.highlightingStyle <- style
+}
