@@ -661,17 +661,6 @@ func pagerFromArgs(
 		panic("Invariant broken: stdout is not a terminal")
 	}
 
-	screen, err := newScreen(*mouseMode, *terminalColorsCount)
-	if err != nil {
-		// Ref: https://github.com/walles/moar/issues/149
-		log.Debug("Failed to set up screen for paging, pumping to stdout instead: ", err)
-		err := pumpToStdout(flagSet.Args()...)
-		if err != nil {
-			return nil, nil, chroma.Style{}, nil, err
-		}
-		return nil, nil, chroma.Style{}, nil, nil
-	}
-
 	if len(flagSet.Args()) > 1 {
 		fmt.Fprintln(os.Stderr, "ERROR: Expected exactly one filename, or data piped from stdin")
 		fmt.Fprintln(os.Stderr)
@@ -690,6 +679,37 @@ func pagerFromArgs(
 		formatter = formatters.TTY16m
 	}
 
+	var reader *m.Reader
+	if stdinIsRedirected {
+		// Display input pipe contents
+		reader = m.NewReaderFromStreamWithoutStyle("", os.Stdin, formatter, *lexer)
+	} else {
+		// Display the input file contents
+		if len(flagSet.Args()) != 1 {
+			panic("Invariant broken: Expected exactly one filename")
+		}
+		reader, err = m.NewReaderFromFilenameWithoutStyle(flagSet.Args()[0], formatter, *lexer)
+		if err != nil {
+			return nil, nil, chroma.Style{}, nil, err
+		}
+	}
+
+	// If the user is doing "sudo something | moar" we can't show the UI until
+	// we start getting data, otherwise we'll mess up sudo's password prompt.
+	reader.AwaitFirstByte()
+
+	// We got the first byte, this means sudo is done (if it was used) and we
+	// can set up the UI.
+	screen, err := newScreen(*mouseMode, *terminalColorsCount)
+	if err != nil {
+		// Ref: https://github.com/walles/moar/issues/149
+		log.Debug("Failed to set up screen for paging, pumping to stdout instead: ", err)
+
+		reader.PumpToStdout()
+
+		return nil, nil, chroma.Style{}, nil, nil
+	}
+
 	var style chroma.Style
 	if *styleOption == nil {
 		t0 := time.Now()
@@ -724,21 +744,7 @@ func pagerFromArgs(
 	} else {
 		style = **styleOption
 	}
-
-	var reader *m.Reader
-	if stdinIsRedirected {
-		// Display input pipe contents
-		reader = m.NewReaderFromStream("", os.Stdin, style, formatter, *lexer)
-	} else {
-		// Display the input file contents
-		if len(flagSet.Args()) != 1 {
-			panic("Invariant broken: Expected exactly one filename")
-		}
-		reader, err = m.NewReaderFromFilename(flagSet.Args()[0], style, formatter, *lexer)
-		if err != nil {
-			return nil, nil, chroma.Style{}, nil, err
-		}
-	}
+	reader.SetStyleForHighlighting(style)
 
 	pager := m.NewPager(reader)
 	pager.WrapLongLines = *wrap
